"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const child_process_1 = __importDefault(require("child_process"));
const concat_stream_1 = __importDefault(require("concat-stream"));
const flamebearer_1 = __importDefault(require("./flamebearer"));
const metrics_1 = require("./metrics");
;
const tickprocessor = require.resolve('../tickprocessor');
function processProfiles(profiles, config) {
    return __awaiter(this, void 0, void 0, function* () {
        const processedScenarios = {};
        // Serialize a bunch of async generation of flamegraphs
        // TODO: need an API story here. how will users get output? data structures? files? both?
        for (const scenarioName of Object.keys(profiles)) {
            const outFile = path_1.default.join(config.outDir, `${scenarioName}`);
            const outFileBaseline = path_1.default.join(config.outDir, `${scenarioName}_base`);
            const result = profiles[scenarioName];
            const processedScenario = yield processProfile(result, outFile);
            if (result.baseline) {
                processedScenario.baseline = yield processProfile(result.baseline, outFileBaseline);
            }
            processedScenarios[scenarioName] = processedScenario;
        }
        return processedScenarios;
    });
}
exports.processProfiles = processProfiles;
function processProfile(profile, outFile) {
    return __awaiter(this, void 0, void 0, function* () {
        return generateFlamegraph(profile, outFile);
    });
}
/**
 * Takes in a v8 or node generated profile log and turn into flamegraph with flamebearer.
 * Generates files based on scenarioName: flamegraph (.html), JS data (.data.js), and error output (.err.txt)
 *
 * @param {string} logFile Profile log file path and base name input.
 * @param {string} outFile File path and base name for file output. Extensions will be added for each type of file output.
 * @returns {Promise<OutputFiles>} Promise resolving to generated files.
 */
function generateFlamegraph(profile, outFile) {
    return __awaiter(this, void 0, void 0, function* () {
        // TODO: account for --windows, --unix and --mac arguments? what is the output difference with and without these args?
        // TODO: use cli
        const preprocessProc = child_process_1.default.spawn(process.execPath, [tickprocessor, '--preprocess', '-j', profile.logFile]);
        const flamegraphFile = outFile + '.html';
        const dataFile = outFile + '.data.js';
        const errorFile = outFile + '.err.txt';
        // TODO: return early?
        if (!fs_1.default.existsSync(path_1.default.dirname(outFile))) {
            console.error(`Directory does not exist: ${path_1.default.dirname(outFile)}`);
        }
        // TODO: this just silently fails if the output directory doesn't exist
        const flamebearerPromise = new Promise((resolve, reject) => {
            const outStream = concat_stream_1.default(preprocessed => {
                if (preprocessed.length > 0) {
                    // This line writes intermediate streaming output for troubleshooting:
                    // fs.writeFileSync(flamegraphfile.split('.html').join('.preprocessed.log'), preprocessed);
                    let [flamegraph, data] = flamebearer_1.default(preprocessed);
                    flamegraph = metrics_1.addMetrics(flamegraph, profile.metrics);
                    fs_1.default.writeFileSync(flamegraphFile, flamegraph);
                    fs_1.default.writeFileSync(dataFile, data);
                    resolve({ output: { dataFile, flamegraphFile } });
                }
            });
            const errStream = concat_stream_1.default(err => {
                if (err.length > 0) {
                    fs_1.default.writeFileSync(errorFile, err);
                    resolve({ error: { errorFile } });
                }
            });
            preprocessProc.stderr.pipe(errStream);
            preprocessProc.stdout.pipe(outStream);
        });
        return flamebearerPromise;
    });
}
/**
 * Takes in a v8 or node generated profile log and generate a summary text file (.summary.log)
 *
 * @param {string} logfile Log file input.
 * @param {string} outfile File path and base name for file output. Extensions will be added for each type of file output.
 * @returns {string} Summary file path.
 */
function generateSummary(logfile, outfile) {
    return __awaiter(this, void 0, void 0, function* () {
        // TODO: account for --windows, --unix and --mac arguments? what is the output difference with and without these args?
        // TODO: use cli
        const summaryProc = child_process_1.default.spawn(process.execPath, [tickprocessor, logfile]);
        const summaryFile = outfile + '.summary.log';
        const flamebearerPromise = new Promise((resolve, reject) => {
            const concatStream = concat_stream_1.default(preprocessed => {
                // TODO: this code just blindly assumes flamegraphfile has an html extension
                fs_1.default.writeFileSync(summaryFile, preprocessed);
                resolve(summaryFile);
            });
            summaryProc.stdout.pipe(concatStream);
        });
        return flamebearerPromise;
    });
}
exports.__unitTestHooks = {
    processProfile
};
