"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const puppeteer_1 = __importDefault(require("puppeteer"));
const util_1 = require("../util");
;
;
// const extraV8Flags = '--log-source-code --log-timer-events';
// const extraV8Flags = '--log-source-code';
const extraV8Flags = '';
/**
 * Profiles scenarios and generates profile (V8) logs.
 *
 * @param config
 * @param scenarios
 */
function profile(scenarios, config) {
    return __awaiter(this, void 0, void 0, function* () {
        const { tempDir, pageActions } = config;
        const logFile = path_1.default.join(tempDir, '/puppeteer.log');
        console.log(`profile logFile: ${logFile}`);
        const browser = yield puppeteer_1.default.launch({
            headless: true,
            // TODO: use --single-process arg?
            args: [
                '--flag-switches-begin',
                '--no-sandbox',
                '--js-flags=--logfile=' + logFile + ' --prof --jitless --no-opt ' + extraV8Flags,
                '--flag-switches-end'
            ]
        });
        // A log file is made for each new tab/page.
        // Make an initial page here to force generation of initial system logs that we are not concerned with.
        // This allows us to associate newly created log files with tests as we run each scenario in a new tab.
        yield browser.newPage();
        // TODO: Need to decide whether it's ok to run tests in parallel. Variance from results seems to indicate
        // not, but then again other things outside of our control will also affect CPU load and results.
        // Run tests sequentially for now, at least as a chance of getting more consistent results when run locally.
        const profiles = {};
        for (const scenarioName of Object.keys(scenarios)) {
            const scenario = scenarios[scenarioName];
            let profileResults = yield profileUrl(browser, scenario.scenario, scenarioName, tempDir, pageActions);
            if (scenario.baseline) {
                profileResults.baseline = yield profileUrl(browser, scenario.baseline, scenarioName, tempDir, pageActions);
            }
            profiles[scenarioName] = profileResults;
        }
        // Clean up
        yield browser.close();
        return profiles;
    });
}
exports.profile = profile;
/**
 * Run profiler against provided URL and return resulting profiler output.
 *
 * @param {*} browser Launched puppeteer instance.
 * @param {string} testUrl Base URL supporting 'scenario' and 'iterations' query parameters.
 * @param {string} profileName Name of scenario that will be used with baseUrl.
 * @param {string} logDir Absolute path to output log profiles.
 * @param {PageActions} pageActions Async opertaion that is executed before taking metrics.
 * @returns {string} Log file path associated with test.
 */
function profileUrl(browser, testUrl, profileName, logDir, pageActions) {
    return __awaiter(this, void 0, void 0, function* () {
        const logFilesBefore = fs_1.default.readdirSync(logDir);
        const page = yield browser.newPage();
        const logFilesAfter = fs_1.default.readdirSync(logDir);
        const testLogFile = util_1.arr_diff(logFilesBefore, logFilesAfter);
        if (testLogFile.length !== 1) {
            // We have to be able to identify log file associated with tab. Throw error if we can't.
            // TODO: what should be the standard for erroring? console.error? throw? return failure?
            // TODO: make sure all async function calls have catch blocks
            // TODO: make sure invalid URLs (and other inputs) don't hang result
            throw new Error(`Could not determine log file for ${testUrl}. Log files detected: [ ${testLogFile} ]`);
        }
        console.log(`Starting test for ${profileName} at ${testUrl}`);
        console.time('Ran profile in');
        if (pageActions) {
            console.log("Started executing user-defined page operations.");
            yield pageActions(page, { url: testUrl });
            console.log("Finished executing user-defined page operations.");
        }
        else {
            yield page.goto(testUrl);
        }
        console.timeEnd('Ran profile in');
        let metrics = yield page.metrics();
        console.log('testLogFile: ' + testLogFile[0]);
        yield page.close();
        return {
            logFile: path_1.default.join(logDir, testLogFile[0]),
            metrics
        };
    });
}
exports.__unitTestHooks = {
    profileUrl
};
